'２次元熱伝導解析V110の標準ﾓｼﾞｭｰﾙ      ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module Th2DCom
    '２次元および軸対称体の熱解析
    'アイソメトリック四角形要素使用
    '自動メッシュ生成は「ＦＥＭ断面データ」ソフトによる
    '要素の節点番号の入力は、反時計回り

    'V101での改良点・・・データ入力画面で密度だけを変更できない不具合を訂正
    'V102での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V103での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V104での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                    （これまでは最長方向の座標軸を採用）
    'V105での改良点・・・非定常解析で「メモリー使用量」優先の場合に文字コードUTF-8の
    '                    出力バグを修正（後処理で読み込み不可のエラーを防止）
    'V106での改良点・・・構造内の面間の熱放射グループを追加、
    '                    構造図画面で「Ctrl+マウス」の矩形ドラッグによる一括設定に
    '                    おいて、熱放射境界だけ設定できないエラーを修正、
    '                    定常解析では熱放射境界をエラー表示
    'V107での改良点・・・構造図画面で条件設定部の表示を２区分から３区分に改善、
    '                    構造図画面で条件設定部の表示辺幅を拡大
    'V108での改良点・・・サブメニューの中間節点自動計算を廃止
    'V109での改良点・・・解析データのデータチェックに要素共通辺の節点を追加
    'V110での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$                 '計算結果表示データ
    Public Cr$                      'CR+LFのコード
    Public DirOpen$                 '立ち上げ時のディレクトリー名
    Public Fadd$                    '連結するファイル名
    Public Fgraph$                  'ｸﾞﾗﾌﾃﾞｰﾀの出力ﾌｧｲﾙ名
    Public Fm$                      '入力データのファイル名
    Public Fy$                      '計算結果のファイル名
    Public Title$                   'タイトル

    Public Timst As Date            '計算開始時刻

    Public Idcycle As Integer       '出力サイクル間隔
    Public InitCond As Integer      'ﾊﾞｯﾁ処理で計算条件設定の有無（0:なし、1:有り）
    Public InitData As Integer      '初期値データの有無（0:なし、1:有り）
    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力、2:FEM2Dﾌｧｲﾙ入力）
    Public Kaxis As Integer         '作図のＸＹ軸の入換え（0:なし、1:有り）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public KcomSide As Integer      '要素辺の表示有無（0:なし、1:有り）
    Public KcontourPoint As Integer '等高線図で場所･数値表示の有無（0:なし、1:有り）
    Public Kcycle As Integer        '作画指定のサイクル番号
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KDrawDatMod As Integer   '構造図でのﾃﾞｰﾀ修正区分（0:個別、1:一括）
    Public KdumyStart As Integer    'ダミー化開始ステップ番号
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public Kfile As Integer         '出力ファイルのポート番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public Kfinish As Integer       'ﾊﾞｯﾁ処理１回の終了区分（0:未終了、1:終了）
    Public KindBatch As Integer     'ﾊﾞｯﾁ処理の有無（0:なし、1:有り）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public Kindex As Integer        'ﾃｷｽﾄｱﾚｲのｲﾝﾃﾞｯｸｽ
    Public KindPost As Integer      '後処理の種類（1:等温線図、2:等温線図ｽﾗｲﾄﾞｼｮｰ、3:主要値表示、
    '                                4:非定常のｸﾞﾗﾌﾃﾞｰﾀ）
    Public KindPref As Integer      '優先選択（0:計算時間、1:ﾒﾓﾘｰ使用量）
    Public KmatDumy As Integer      'ダミー化の材料番号
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseDset As Integer    'ﾏｳｽ指定要素のﾃﾞｰﾀ項目（1:材料番号、2:温度固定節点、3:熱流束、
    '                                4:熱伝達、5:熱放射、6:内部発熱）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KnewUFileB As Integer    '最近使用ﾌｧｲﾙBの更新有無
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public KStage As Integer        'ﾃﾞｰﾀ入力の段階番号
    Public Kstep As Integer         '作画指定のステップ番号
    Public Ktime As Integer         '出力メッシュ番号
    Public Mprop As Integer         '材料データの成分数(=3)
    Public Mstat As Integer         '解析種別（０:定常、１:非定常）
    Public Nall As Integer          '変数の総数
    Public Namb, Nambb As Integer   '熱伝達境界の辺の数
    Public Nbound, Nboundb As Integer   '温度固定節点数
    Public Ncycle As Integer        '境界条件のサイクル数
    Public Ndel As Integer          '削除要素数
    Public Ndim As Integer          '１節点当りの変数の数(=1)
    Public NDisp As Integer         '等高線の分割数
    Public Nelem As Integer         '要素数
    Public Nflux, Nfluxb As Integer '熱流束境界の辺の数
    Public Ngaus As Integer         'ガウス積分点数(3)
    Public Nline As Integer         'ﾌｧｲﾙの行数
    Public Nmat As Integer          '材料データ数
    Public NMdivMax As Integer      '材料の物性区分数最大値
    Public NPgraph As Integer       'ｸﾞﾗﾌﾃﾞｰﾀの節点数
    Public Npoint As Integer        '節点数
    Public Nqint, Nqintb As Integer '内部発熱の要素数
    Public Nrad, Nradb As Integer   '熱放射境界の辺の数

    Public NradG, NradGb As Integer '熱放射グループの数
    Public NumRmax As Integer       '熱放射グループの最大辺数

    Public Nstep As Integer         '境界条件のステップ数
    Public Ntype As Integer         '問題のタイプ（2:２次元、4:軸対称）
    Public Ntime As Integer         '非定常の出力時間メッシュ数
    Public Nvar As Integer          '１要素当りの変数の総数（=8）

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public DtempMax As Single       '非定常の計算1回の温度変化許容値
    Public Dtime As Single          '非定常の出力の計算時間間隔
    Public DtimeInit As Single      '非定常の計算時間間隔の初期値
    Public RateDumy As Single       'ダミー化の熱容量比率
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public SigmaG As Single         '熱放射グループのｽﾃﾌｧﾝ･ﾎﾞﾙﾂﾏﾝ定数（共通）
    Public TimeMax As Single        '非定常の打切り時間
    Public TimeNew As Single        '現在の計算時刻
    Public VdispMax As Single       '作画変数の最高値
    Public VdispMin As Single       '作画変数の最低値
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public Yupper As Single         '作画領域のY最大値

    Public UFileB(6) As String      '最近使用した解析ﾌｧｲﾙの保存配列

    Public KAppPoint() As Integer   'ﾃﾞｰﾀ設定の節点
    Public KAppSide(,) As Integer   'ﾃﾞｰﾀ設定の要素指定辺
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KdrawElemOld() As Integer    '変更前の各要素の作画有無（0:なし、1:有り）
    Public KEInside(,) As Integer   '要素の各辺の内部区分（0:外周、1:内部）
    Public Kelemq() As Integer      '内部発熱の要素番号
    Public Kpamb(,) As Integer      '熱伝達境界の辺の３節点番号
    Public Koption(3) As Integer    'オプション有無（1:バックアップ、
    '                                2:要素入力画面の節点番号表示、3:節点入力画面の表示）
    Public Kpflux(,) As Integer     '熱流束境界の３節点番号
    Public KPgraph(20) As Integer   'ｸﾞﾗﾌ節点番号
    Public Kprad(,) As Integer      '熱放射境界の辺の３節点番号

    Public MemRad(,,) As Integer    '熱放射グループの辺の３節点番号
    Public NumRad() As Integer      '熱放射グループの辺の数

    Public NMdiv() As Integer       '材料の物性区分数
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public Nrest() As Integer       '温度固定の節点番号
    Public NumMat() As Integer      '各要素の材料種類番号

    Public Alpha(,) As Single       '熱伝達係数
    Public Cspec(,) As Single       '材料の比熱

    Public EpsRad(,) As Single      '熱放射グループの辺の放射率

    Public Pnorm(,) As Single       '単位面積当りの垂直熱流束
    Public PosGaus(3) As Single     'ｶﾞｳｽ積分点の位置(於要素座標）
    Public Qint(,) As Single        '内部発熱率（単位体積、単位時間当り）

    Public RadRate(,) As Single     '熱放射の係数(ｽﾃﾌｧﾝﾎﾞﾙﾂﾏﾝ定数×放射率×形状係数)
    Public Rlamda(,) As Single      '材料の熱伝導率
    Public Rou() As Single          '材料の密度
    Public TempAmb(,) As Single     '熱伝達の外部温度
    Public TempFix(,) As Single     '固定温度
    Public TempInit() As Single     '初期温度
    Public TempMat(,) As Single     '材料の区分上限温度
    Public TempRad(,) As Single     '熱放射源温度
    Public TimeStep() As Single     'ステップ時間
    Public Tu() As Single           'FEM計算の解
    Public WeightG(3) As Single     'ｶﾞｳｽ積分点の重み
    Public X(), Y() As Single       '節点座標
    Public Xdraw(8), Ydraw(8) As Single     '色塗り多角形の座標
    Public XE(8), YE(8) As Single   '要素計算の配列
    Public Xmouse(), Ymouse() As Single     'ﾏｳｽ位置座標

    Sub AmbDupCheck(ByRef nd As Integer)
        '熱伝達の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Namb) As Integer

        nd = 0
        For i = 1 To Namb - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Namb
                    If Kpamb(i, 1) = Kpamb(ii, 1) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Nstep
                            Alpha(i, j) = Alpha(ii, j)
                            TempAmb(i, j) = TempAmb(ii, j)
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Namb, 3) As Integer
            Dim buf2(Namb, Nstep), buf3(Namb, Nstep) As Single
            For i = 1 To Namb
                For j = 1 To 3
                    nbuf1(i, j) = Kpamb(i, j)
                Next j
                For j = 1 To Nstep
                    buf2(i, j) = Alpha(i, j)
                    buf3(i, j) = TempAmb(i, j)
                Next j
            Next i
            nb = Namb - nd
            ReDim Kpamb(nb, 3), Alpha(nb, Nstep), TempAmb(nb, Nstep)
            nb = 0
            For i = 1 To Namb
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 3
                        Kpamb(nb, j) = nbuf1(i, j)
                    Next j
                    For j = 1 To Nstep
                        Alpha(nb, j) = buf2(i, j)
                        TempAmb(nb, j) = buf3(i, j)
                    Next j
                End If
            Next i
            Nambb = nb
            Namb = nb
        End If

    End Sub

    Sub AxisCheck(ByRef icheck As Integer, ByRef k1 As Integer, _
        ByRef k2 As Integer, ByRef ratio As Single, ByRef xmin As Single, _
        ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        '         最初に-1が設定された場合は、RZ座標のR最小値は0としない
        'k1,k2･･･要素の開始、終了番号
        'ratio･･･作画直角回転なしの縦横比上限（指定値）
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim i, ie, ik, j As Integer
        Dim b1, b2, ratiob As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = X(i) : xmax = xmin
                        ymin = Y(i) : ymax = ymin
                    Else
                        If X(i) < xmin Then
                            xmin = X(i)
                        ElseIf X(i) > xmax Then
                            xmax = X(i)
                        End If
                        If Y(i) < ymin Then
                            ymin = Y(i)
                        ElseIf Y(i) > ymax Then
                            ymax = Y(i)
                        End If
                    End If
                Next j
            End If
        Next ie
        If Ntype = 4 And icheck >= 0 Then
            xmin = 0
        End If
        b1 = xmax - xmin
        b2 = ymax - ymin
        If b1 = 0 Then
            icheck = 0
        Else
            ratiob = b2 / b1
            If ratiob > ratio Then
                icheck = 1
            Else
                icheck = 0
            End If
        End If

    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf()･･･節点の座標
        Dim i, ie, kind, nb As Integer
        Dim b1, b2, pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        b1 = pa - XaxisMin
        b2 = qa - YaxisMin
        If b2 > b1 Then
            b1 = b2
        End If
        b2 = b1 * 0.00002
        Call RoundPrep(kind, nb, b2)    '丸め準備

        For i = 1 To Npoint
            xbuf(i) = X(i)
            ybuf(i) = Y(i)
        Next i
        For i = 1 To Npoint
            If xbuf(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If ybuf(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'kcbuf()･･･元の節点の変数順序
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        Dim i, ie, ii, iii, j, jj, k, kb, mx As Integer

        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To 7
                i = NNelem(ie, ii)
                For jj = ii + 1 To 8
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie

        GoTo Skip10
        If NradG > 0 Then
            For i = 1 To NradG
                For j = 1 To NumRad(i) - 1
                    For ii = 1 To 3
                        kb = kcbuf(MemRad(i, j, ii))
                        For jj = j + 1 To NumRad(i)
                            For iii = 1 To 3
                                k = Math.Abs(kb - kcbuf(MemRad(i, jj, iii)))
                                If k > mx Then mx = k
                            Next iii
                        Next jj
                    Next ii
                Next j
            Next i
        End If
Skip10:

        mb = Ndim * (mx + 1)

    End Sub

    Sub BoundDupCheck(ByRef nd As Integer)
        '固定温度節点重複のﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nbound) As Integer

        nd = 0
        For i = 1 To Nbound - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nbound
                    If Nrest(i) = Nrest(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Nstep
                            TempFix(i, j) = TempFix(ii, j)
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nbound) As Integer
            Dim buf2(Nbound, Nstep) As Single
            For i = 1 To Nbound
                nbuf1(i) = Nrest(i)
                For j = 1 To Nstep
                    buf2(i, j) = TempFix(i, j)
                Next j
            Next i
            nb = Nbound - nd
            ReDim Nrest(nb), TempFix(nb, Nstep)
            nb = 0
            For i = 1 To Nbound
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    Nrest(nb) = nbuf1(i)
                    For j = 1 To Nstep
                        TempFix(nb, j) = buf2(i, j)
                    Next j
                End If
            Next i
            Nbound = nb
            Nboundb = nb
        End If

    End Sub

    Sub CenterLine(ByVal gr As Graphics, ByVal icheck As Integer)
        '中心線の作画
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'kind･･･ﾋﾟｸﾁｬｰﾎﾞｯｸｽの長さ種類（0:標準、1:大）
        Dim myPen As New Pen(Color.Black)
        myPen.DashStyle = Drawing2D.DashStyle.DashDot
        Select Case icheck
            Case 0
                gr.DrawLine(myPen, Xlower, Ylower, Xlower, Yupper)
            Case 1
                gr.DrawLine(myPen, Xlower, Ylower, Xupper, Ylower)
        End Select

    End Sub

    Sub CondTempCheck(ByRef ik As Integer, ByRef tempcmax As Single, _
    ByRef tempcmin As Single)
        '境界条件の温度ﾁｪｯｸ
        'ik･･･境界の有無（0:なし、1:有り）
        'tempcmax･･･最高温度
        'tempcmin･･･最低温度
        Dim i, ic, j As Integer
        Dim b1 As Single

        ik = 0
        ic = 0
        If Nbound > 0 Then
            ik = 1
            For i = 1 To Nbound
                For j = 1 To Nstep
                    b1 = TempFix(i, j)
                    Call TempComp(ic, b1, tempcmax, tempcmin) '温度比較
                Next j
            Next i
        End If

        If Namb > 0 Then
            ik = 1
            For i = 1 To Namb
                For j = 1 To Nstep
                    b1 = TempAmb(i, j)
                    Call TempComp(ic, b1, tempcmax, tempcmin) '温度比較
                Next j
            Next i
        End If

        If Nrad > 0 Then
            ik = 1
            For i = 1 To Nrad
                For j = 1 To Nstep
                    b1 = TempRad(i, j)
                    Call TempComp(ic, b1, tempcmax, tempcmin) '温度比較
                Next j
            Next i
        End If

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim aa$, aerr1$
        Dim i, ib, ie, im, ip, j, jb, k, kb, kh As Integer
        Dim n1, n2, n3, nb, nx As Integer
        Dim b1, b2 As Single
        Dim abuf$(10)
        aerr1$ = "になっています！"
        num = 0

        If Ntype <> 2 And Ntype <> 4 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  問題のタイプの番号が間違っています！"
        End If
        If Npoint < 8 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  節点数が不足です！"
        End If
        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If Nmat < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  材料データ数が" + Str$(Nmat) + aerr1$
        End If
        If Nstep < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  時間ステップ数が" + Str$(Nstep) + aerr1$
        End If
        If Nbound < 1 Then
            If Nflux < 1 And Namb < 1 And Nrad < 1 And Nqint < 1 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  境界条件が設定されていません！"
            End If
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            nb = NumMat(ie)
            If nb < 1 Or nb > Nmat Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf$(num) = aa$ + " 材料番号が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            For j = 1 To 8
                nb = NNelem(ie, j)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "の"
                    aa$ = aa$ + " 節点番号" + Str$(nb)
                    abuf$(num) = aa$ + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If j < 8 Then
                    For k = j + 1 To 8
                        nx = NNelem(ie, k)
                        If nb = nx Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                            abuf$(num) = aa$ + " ２つの節点番号が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    Next k
                End If
            Next j
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = X(ip) - X(j)
                b2 = Y(ip) - Y(j)
                If b1 = 0 And b2 = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j) + "が"
                    abuf$(num) = aa$ + " 同じ座標位置" + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        '不要節点のﾁｪｯｸ
        Dim kbuf(Npoint) As Integer
        For ie = 1 To Nelem
            For j = 1 To 8
                kbuf(NNelem(ie, j)) = 1
            Next j
        Next ie
        For ip = 1 To Npoint
            If kbuf(ip) = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が使われていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        Next ip

        '要素共通辺のﾁｪｯｸ
        ReDim kbuf(Npoint)
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                ib = NNelem(ie, j)
                If kbuf(ib) = 0 Then
                    kbuf(ib) = 1    '頂点節点に指定
                End If
            Next j
        Next ie
        For ie = 1 To Nelem
            For j = 2 To 8 Step 2
                ib = NNelem(ie, j)
                If kbuf(ib) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ib) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ie

        If Nbound > 0 Then     '温度固定の節点ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nbound
                nb = Nrest(ib)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  温度固定節点の固定番号" + Str$(ib) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        '材料ﾃﾞｰﾀのﾁｪｯｸ
        For im = 1 To Nmat
            nb = NMdiv(im)
            If nb < 1 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                abuf$(num) = aa$ + " 物性区分数が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            If Rou(im) <= 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                abuf$(num) = aa$ + " 密度が" + Str$(Rou(im)) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            For j = 1 To nb
                b1 = Cspec(im, j)
                If b1 <= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                    abuf$(num) = aa$ + " 比熱が" + Str$(b1) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                b1 = Rlamda(im, j)
                If b1 <= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                    abuf$(num) = aa$ + " 熱伝導率が" + Str$(b1) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If j > 1 Then
                    b1 = TempMat(im, j) - TempMat(im, j - 1)
                    If b1 <= 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                        abuf$(num) = aa$ + " 上限温度が上がっていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                End If
            Next j
        Next im

        If Nstep > 1 Then   'ｽﾃｯﾌﾟ時間ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nstep
                If TimeStep(ib) <= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  ステップ番号" + Str$(ib) + "の"
                    abuf$(num) = aa$ + " 時間が" + Str$(TimeStep(i)) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        Dim i1, i2, ic, ii, j1, j2, jj, js As Integer
        Dim kmulti(Nelem, 8) As Integer
        For ie = 1 To Nelem
            i1 = NNelem(ie, 8)
            For j = 1 To 8
                i2 = NNelem(ie, j)
                For ii = ie + 1 To Nelem
                    j1 = NNelem(ii, 8)
                    For jj = 1 To 8
                        j2 = NNelem(ii, jj)
                        Call NumCheck(ic, i1, i2, j1, j2)   '要素辺の重なりﾁｪｯｸ
                        If ic = 1 Then
                            kmulti(ie, j) = 1
                            kmulti(ii, jj) = 1
                        End If
                        j1 = j2
                    Next jj
                Next ii
                i1 = i2
            Next j
        Next ie
        If Nflux > 0 Then   '熱流束境界のﾁｪｯｸ
            For ib = 1 To Nflux
                For j = 1 To 3
                    jb = Kpflux(ib, j)
                    If jb < 1 Or jb > Npoint Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱流束境界の番号" + Str$(ib) + "の"
                        abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    If j < 3 Then
                        For k = j + 1 To 3
                            kb = Kpflux(ib, k)
                            If jb = kb Then
                                num = num + 1
                                aa$ = "Error" + LTrim$(Str$(num))
                                aa$ = aa$ + "  熱流束境界の番号" + Str$(ib) + "で"
                                abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "が重複しています！"
                                If num = 10 Then
                                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                    Exit Sub
                                End If
                            End If
                        Next k
                    End If
                Next j
                n1 = Kpflux(ib, 1)
                n2 = Kpflux(ib, 2)
                n3 = Kpflux(ib, 3)
                Call SideSearch(kh, js, n1, n2, n3) '辺の探索
                If kh = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱流束境界の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 辺の指定が間違っています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                ElseIf kmulti(kh, js) = 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱流束境界の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 内部辺が指定されています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Namb > 0 Then    '熱伝達ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Namb
                For j = 1 To 3
                    jb = Kpamb(ib, j)
                    If jb < 1 Or jb > Npoint Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱伝達データの番号" + Str$(ib) + "の"
                        abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    If j < 3 Then
                        For k = j + 1 To 3
                            kb = Kpamb(ib, k)
                            If jb = kb Then
                                num = num + 1
                                aa$ = "Error" + LTrim$(Str$(num))
                                aa$ = aa$ + "  熱伝達データの番号" + Str$(ib) + "で"
                                abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "が重複しています！"
                                If num = 10 Then
                                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                    Exit Sub
                                End If
                            End If
                        Next k
                    End If
                Next j
                n1 = Kpamb(ib, 1)
                n2 = Kpamb(ib, 2)
                n3 = Kpamb(ib, 3)
                Call SideSearch(kh, js, n1, n2, n3) '辺の探索
                If kh = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱伝達データの番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 辺の指定が間違っています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                ElseIf kmulti(kh, js) = 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱伝達データの番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 内部辺が指定されています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nrad > 0 Then    '熱放射ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nrad
                For j = 1 To 3
                    jb = Kprad(ib, j)
                    If jb < 1 Or jb > Npoint Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱放射データの番号" + Str$(ib) + "の"
                        abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    If j < 3 Then
                        For k = j + 1 To 3
                            kb = Kprad(ib, k)
                            If jb = kb Then
                                num = num + 1
                                aa$ = "Error" + LTrim$(Str$(num))
                                aa$ = aa$ + "  熱放射データの番号" + Str$(ib) + "で"
                                abuf$(num) = aa$ + " 節点番号" + Str$(jb) + "が重複しています！"
                                If num = 10 Then
                                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                    Exit Sub
                                End If
                            End If
                        Next k
                    End If
                Next j
                n1 = Kprad(ib, 1)
                n2 = Kprad(ib, 2)
                n3 = Kprad(ib, 3)
                Call SideSearch(kh, js, n1, n2, n3) '辺の探索
                If kh = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱放射データの番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 辺の指定が間違っています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                ElseIf kmulti(kh, js) = 1 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱放射データの番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 内部辺が指定されています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nqint > 0 Then   '内部発熱ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nqint
                nb = Kelemq(ib)
                If nb < 1 Or nb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  内部発熱の番号" + Str$(ib) + "で"
                    abuf$(num) = aa$ + " 要素番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If NradG > 0 Then    '熱放射ｸﾞﾙｰﾌﾟのﾁｪｯｸ
            For ib = 1 To NradG
                For k = 1 To NumRad(ib)
                    n1 = MemRad(ib, k, 1)
                    n2 = MemRad(ib, k, 2)
                    n3 = MemRad(ib, k, 3)
                    Call SideSearch(kh, js, n1, n2, n3) '辺の探索
                    If kh = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱放射グループ" + Str$(ib) + "の辺No." + Str(k) + "で"
                        abuf$(num) = aa$ + " 節点が間違っています！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    ElseIf kmulti(kh, js) = 1 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱放射グループ" + Str$(ib) + "の辺No." + Str(k) + "で"
                        abuf$(num) = aa$ + " 内部辺が指定されています！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next k
                For k = 1 To NumRad(ib) - 1
                    n1 = MemRad(ib, k, 2)
                    For kb = k + 1 To NumRad(ib)
                        If n1 = MemRad(ib, kb, 2) Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  熱放射グループ" + Str$(ib) + +"の辺No." + Str(k) + "と"
                            abuf$(num) = aa$ + " 辺No." + Str$(kb) + "が重複しています！"
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    Next kb
                Next k
            Next ib
        End If

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, j, jb, k As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Therm2D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ntype = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nbound = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nstep = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nflux = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Namb = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nrad = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nqint = dat(1)

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) = "熱放射グループの数" Then
            NradG = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
        Else
            NradG = 0
        End If
        Ngaus = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Mprop = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        InitData = dat(1)

        ReDim NNelem(Nelem, 8), NumMat(Nelem)
        ReDim X(Npoint), Y(Npoint)
        ReDim Nrest(Nbound), TempFix(Nbound, Nstep)
        ReDim NMdiv(Nmat), Rou(Nmat), TimeStep(Nstep)
        ReDim Kpflux(Nflux, 3), Pnorm(Nflux, Nstep)
        ReDim Kpamb(Namb, 3), Alpha(Namb, Nstep), TempAmb(Namb, Nstep)
        ReDim Kprad(Nrad, 3), RadRate(Nrad, Nstep), TempRad(Nrad, Nstep)
        ReDim Kelemq(Nqint), Qint(Nqint, Nstep)
        ReDim NumRad(NradG)
        ReDim TempInit(Npoint)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 8
                NNelem(i, j) = dat(j + 1)
            Next j
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            X(i) = dat(1)
            Y(i) = dat(2)
        Next i

        If Nbound > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nbound
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Nrest(i) = dat(1)
                For j = 1 To Nstep
                    TempFix(i, j) = dat(j + 1)
                Next j
            Next i
        End If

        aa$ = sr.ReadLine
        NMdivMax = 0
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NMdiv(i) = dat(1)
            If NMdiv(i) > NMdivMax Then
                NMdivMax = NMdiv(i)
            End If
        Next i
        ReDim Cspec(Nmat, NMdivMax), Rlamda(Nmat, NMdivMax), TempMat(Nmat, NMdivMax)
        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Rou(i) = dat(1)
            aa$ = sr.ReadLine
            For j = 1 To NMdiv(i)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Cspec(i, j) = dat(1)
                Rlamda(i, j) = dat(2)
                TempMat(i, j) = dat(3)
            Next j
        Next i

        If Nstep > 1 Then
            aa$ = sr.ReadLine
            For i = 1 To Nstep
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                TimeStep(i) = dat(1)
            Next i
        End If

        If Nflux > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nflux
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Kpflux(i, 1) = dat(1)
                Kpflux(i, 2) = dat(2)
                Kpflux(i, 3) = dat(3)
                For j = 1 To Nstep
                    Pnorm(i, j) = dat(j + 3)
                Next j
            Next i
        End If

        If Namb > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Namb
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Kpamb(i, 1) = dat(1)
                Kpamb(i, 2) = dat(2)
                Kpamb(i, 3) = dat(3)
                For j = 1 To Nstep
                    jb = 2 * j + 1
                    Alpha(i, j) = dat(jb + 1)
                    TempAmb(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If Nrad > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nrad
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Kprad(i, 1) = dat(1)
                Kprad(i, 2) = dat(2)
                Kprad(i, 3) = dat(3)
                For j = 1 To Nstep
                    jb = 2 * j + 1
                    RadRate(i, j) = dat(jb + 1)
                    TempRad(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If Nqint > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nqint
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Kelemq(i) = dat(1)
                For j = 1 To Nstep
                    Qint(i, j) = dat(j + 1)
                Next j
            Next i
        End If

        If NradG > 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            SigmaG = dat(1)
            aa$ = sr.ReadLine
            For i = 1 To NradG
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NumRad(i) = dat(1)
            Next i
            NumRmax = NumRad(1)
            If NradG > 1 Then
                For i = 2 To NradG
                    If NumRad(i) > NumRmax Then
                        NumRmax = NumRad(i)
                    End If
                Next
            End If
            ReDim MemRad(NradG, NumRmax, 3), EpsRad(NradG, NumRmax)
            For i = 1 To NradG
                aa$ = sr.ReadLine
                aa$ = sr.ReadLine
                For k = 1 To NumRad(i)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    For j = 1 To 3
                        MemRad(i, k, j) = dat(j)
                    Next j
                    EpsRad(i, k) = dat(4)
                Next k
            Next i
        End If

        If InitData > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Npoint
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                TempInit(i) = dat(1)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j, k As Integer

        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "Therm2D"
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "問題のタイプ" + "," + Str(Ntype)
        sw.WriteLine(aa$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        aa$ = "温度固定節点数" + "," + Str(Nbound)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        aa$ = "境界条件のステップ数" + "," + Str(Nstep)
        sw.WriteLine(aa$)
        aa$ = "熱流束境界の辺の数" + "," + Str(Nflux)
        sw.WriteLine(aa$)
        aa$ = "熱伝達境界の辺の数" + "," + Str(Namb)
        sw.WriteLine(aa$)
        aa$ = "熱放射境界の辺の数" + "," + Str(Nrad)
        sw.WriteLine(aa$)
        aa$ = "内部発熱の要素の数" + "," + Str(Nqint)
        sw.WriteLine(aa$)

        aa$ = "熱放射グループの数" + "," + Str(NradG)
        sw.WriteLine(aa$)

        aa$ = "ガウス積分点数" + "," + Str(Ngaus)
        sw.WriteLine(aa$)
        Ndim = 1 : Mprop = 3
        aa$ = "１節点当り変数の数" + "," + Str(Ndim)
        sw.WriteLine(aa$)
        aa$ = "材料データの成分数" + "," + Str(Mprop)
        sw.WriteLine(aa$)
        aa$ = "初期値データの有無" + "," + Str(InitData)
        sw.WriteLine(aa$)

        aa$ = "要素データ (要素番号、材料番号、8節点番号)"
        sw.WriteLine(aa$)
        For i = 1 To Nelem
            aa$ = Str$(i) + ",   " + Str$(NumMat(i))
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        If Ntype = 2 Then
            aa$ = "節点データ（節点番号、Ｘ座標、Ｙ座標）"
        Else
            aa$ = "節点データ（節点番号、Ｒ座標、Ｚ座標）"
        End If
        sw.WriteLine(aa$)
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            sw.WriteLine(aa$)
        Next i

        If Nbound > 0 Then
            aa$ = "温度固定境界 (固定番号、節点番号、各ｽﾃｯﾌﾟの温度)"
            sw.WriteLine(aa$)
            For i = 1 To Nbound
                aa$ = Str$(i) + ",   " + Str$(Nrest(i))
                For j = 1 To Nstep
                    aa$ = aa$ + ",   " + Str$(TempFix(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        aa$ = "各材料の物性区分数（材料番号、区分数)"
        sw.WriteLine(aa$)
        For i = 1 To Nmat
            aa$ = "  " + Str$(i) + ",  " + Str$(NMdiv(i))
            sw.WriteLine(aa$)
        Next i
        sw.WriteLine("各材料の物性データ")
        For i = 1 To Nmat
            aa$ = "材料番号" + "," + Str(i)
            sw.WriteLine(aa$)
            aa$ = "  密度" + "," + Str(Rou(i))
            sw.WriteLine(aa$)
            sw.WriteLine("  区分番号、比熱、熱伝導率、上限温度")
            For j = 1 To NMdiv(i)
                aa$ = "    " + Str$(j) + ",  " + Str$(Cspec(i, j))
                aa$ = aa$ + ",  " + Str$(Rlamda(i, j)) + ",  " + Str$(TempMat(i, j))
                sw.WriteLine(aa$)
            Next j
        Next i

        If Nstep > 1 Then
            aa$ = "境界条件のステップ時間（ステップ番号、時間）"
            sw.WriteLine(aa$)
            For i = 1 To Nstep
                aa$ = Str$(i) + ",  " + Str$(TimeStep(i))
                sw.WriteLine(aa$)
            Next i
        End If

        If Nflux > 0 Then
            aa$ = "熱流束データ（熱流束番号、辺の３節点番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの熱流束（単位面積当り））"
            sw.WriteLine(aa$)
            For i = 1 To Nflux
                aa$ = Str$(i)
                For j = 1 To 3
                    aa$ = aa$ + ",   " + Str$(Kpflux(i, j))
                Next j
                For j = 1 To Nstep
                    aa$ = aa$ + ",   " + Str$(Pnorm(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Namb > 0 Then
            aa$ = "熱伝達データ（熱伝達番号、辺の３節点番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの熱伝達係数・外部温度）"
            sw.WriteLine(aa$)
            For i = 1 To Namb
                aa$ = Str$(i)
                For j = 1 To 3
                    aa$ = aa$ + ",   " + Str$(Kpamb(i, j))
                Next j
                For j = 1 To Nstep
                    aa$ = aa$ + ",   " + Str$(Alpha(i, j))
                    aa$ = aa$ + ",   " + Str$(TempAmb(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nrad > 0 Then
            aa$ = "熱放射データ（熱放射番号、辺の３節点番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの放射係数・外部温度）"
            sw.WriteLine(aa$)
            For i = 1 To Nrad
                aa$ = Str$(i)
                For j = 1 To 3
                    aa$ = aa$ + ",   " + Str$(Kprad(i, j))
                Next j
                For j = 1 To Nstep
                    aa$ = aa$ + ",   " + Str$(RadRate(i, j))
                    aa$ = aa$ + ",   " + Str$(TempRad(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nqint > 0 Then
            aa$ = "内部発熱データ（内部発熱番号、要素番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの内部発熱率）"
            sw.WriteLine(aa$)
            For i = 1 To Nqint
                aa$ = Str$(i) + ",   " + Str$(Kelemq(i))
                For j = 1 To Nstep
                    aa$ = aa$ + ",   " + Str$(Qint(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If NradG > 0 Then
            aa$ = "熱放射グループ共通のステファン・ボルツマン定数" + ",  " + Str(SigmaG)
            sw.WriteLine(aa$)
            aa$ = "熱放射グループ（グループ番号、辺の個数）"
            sw.WriteLine(aa$)
            For i = 1 To NradG
                aa$ = Str(i) + ",  " + Str(NumRad(i))
                sw.WriteLine(aa$)
            Next i
            For i = 1 To NradG
                aa = "＜グループ＞" + Str(i)
                sw.WriteLine(aa$)
                aa$ = "  辺No.、辺の３節点番号、放射率"
                sw.WriteLine(aa$)
                For k = 1 To NumRad(i)
                    aa$ = "  " + Str$(k)
                    For j = 1 To 3
                        aa$ = aa$ + ",  " + Str$(MemRad(i, k, j))
                    Next j
                    aa$ = aa$ + ",  " + Str(EpsRad(i, k))
                    sw.WriteLine(aa$)
                Next k
            Next i
        End If

        If InitData > 0 Then
            aa$ = "初期値データ（節点番号、初期温度）"
            sw.WriteLine(aa$)
            For i = 1 To Npoint
                aa$ = Str$(i) + ",  " + Str$(TempInit(i))
                sw.WriteLine(aa$)
            Next i
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawNodeVar(ByRef gr As Graphics, ByVal kind As Integer, _
    ByVal aa As String, ByVal xb As Single, ByVal yb As Single)
        '節点変数の描画
        'kind･･･種別（0:基本、1:灰色）
        'aa･･･変数のテキスト
        'xb,yb･･･描画位置
        Dim xbw, ybw As Single
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", 10, FontStyle.Bold)

        xbw = xb - 1
        ybw = yb
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb
        ybw = yb - 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        ybw = yb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        If kind = 0 Then
            gr.DrawString(aa$, fnt2, Brushes.Purple, xb, yb)
        Else
            gr.DrawString(aa$, fnt2, Brushes.Gray, xb, yb)
        End If

    End Sub

    Sub DrawScale(ByRef icheck As Integer, ByVal dpixb As Single, _
        ByVal dpiyb As Single, ByVal xlen As Single, ByVal ylen As Single, _
        ByVal xmin As Single, ByVal xmax As Single, _
        ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb
        b1 = xmax - xmin : b2 = ymax - ymin
        Select Case icheck
            Case 0
                If Cfix < 1 Then
                    If b1 <> 0 Then
                        cx = xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2 / Cfix
                    End If
                Else
                    If b1 <> 0 Then
                        cx = Cfix * xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2
                    End If
                End If
                If b1 = 0 Then
                    XYscale = cy
                ElseIf b2 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
            Case 1
                If Cfix < 1 Then
                    If b2 <> 0 Then
                        cx = xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1 / Cfix
                    End If
                Else
                    If b2 <> 0 Then
                        cx = Cfix * xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1
                    End If
                End If
                If b2 = 0 Then
                    XYscale = cy
                ElseIf b1 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
        End Select

    End Sub

    Sub Elem2DPrep()
        '構造図作画準備
        Dim ie As Integer
        Dim pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        KE1 = 1
        KE2 = Nelem
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        Call OutSideCheck() '外周辺のﾁｪｯｸ

    End Sub

    Sub ElemSearch2(ByRef k1 As Integer, ByRef k2 As Integer)
        '拡大作画する要素の探索
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        Dim ib, ic, ie, j, kmax, kmin As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    b1 = X(ib) - Xmouse(1)
                    b2 = X(ib) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        If Kaxis = 0 Then
                            b1 = Y(ib) - Ymouse(2)
                            b2 = Y(ib) - Ymouse(1)
                        Else
                            b1 = Y(ib) - Ymouse(1)
                            b2 = Y(ib) - Ymouse(2)
                        End If
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 8 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If
    End Sub

    Sub ElemSearch3(ByVal k1 As Integer, ByVal k2 As Integer, ByRef nb As Integer)
        '矩形領域の辺の探索
        'k1,k2･･･要素の開始、終了番号
        'nb･･･適合辺の数
        Dim ib, ie, j, jj, k As Integer
        Dim b1, b2 As Single
        Dim kbuf(3) As Integer

        nb = 0
        For ie = 1 To Nelem
            For j = 1 To 4
                KAppSide(ie, j) = 0
            Next j
        Next ie
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    If KEInside(ie, j) = 0 Then
                        jj = 2 * j - 1
                        kbuf(1) = NNelem(ie, jj)
                        kbuf(2) = NNelem(ie, jj + 1)
                        jj = jj + 2
                        If jj > 8 Then
                            jj = 1
                        End If
                        kbuf(3) = NNelem(ie, jj)
                        For k = 1 To 3
                            ib = kbuf(k)
                            b1 = X(ib) - Xmouse(1)
                            b2 = X(ib) - Xmouse(2)
                            If b1 >= 0 And b2 <= 0 Then
                                If Kaxis = 0 Then
                                    b1 = Y(ib) - Ymouse(2)
                                    b2 = Y(ib) - Ymouse(1)
                                Else
                                    b1 = Y(ib) - Ymouse(1)
                                    b2 = Y(ib) - Ymouse(2)
                                End If
                                If b1 >= 0 And b2 <= 0 Then
                                    If k = 3 Then
                                        KAppSide(ie, j) = 1
                                        nb = nb + 1
                                    End If
                                Else
                                    Exit For
                                End If
                            Else
                                Exit For
                            End If
                        Next k
                    End If
                Next j
            End If
        Next ie

    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByRef abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub FluxDupCheck(ByRef nd As Integer)
        '熱流束の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nflux) As Integer

        nd = 0
        For i = 1 To Nflux - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nflux
                    If Kpflux(i, 1) = Kpflux(ii, 1) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Nstep
                            Pnorm(i, j) = Pnorm(ii, j)
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nflux, 3) As Integer
            Dim buf2(Nflux, Nstep) As Single
            For i = 1 To Nflux
                For j = 1 To 3
                    nbuf1(i, j) = Kpflux(i, j)
                Next j
                For j = 1 To Nstep
                    buf2(i, j) = Pnorm(i, j)
                Next j
            Next i
            nb = Nflux - nd
            ReDim Kpflux(nb, 3), Pnorm(nb, Nstep)
            nb = 0
            For i = 1 To Nflux
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 3
                        Kpflux(nb, j) = nbuf1(i, j)
                    Next j
                    For j = 1 To Nstep
                        Pnorm(nb, j) = buf2(i, j)
                    Next j
                End If
            Next i
            Nfluxb = nb
            Nflux = nb
        End If

    End Sub

    Sub MiniSort2(ByVal kind As Integer, ByRef nsort() As Integer, _
    ByVal work() As Single, ByVal xbuf() As Single, ByVal ybuf() As Single)
        'ﾐﾆｿｰﾄ2
        '同じ変数値の場合に、第2座標値でソート
        'kind･･･主ｿｰﾄの座標軸番号（1:X軸、2:Y軸）
        'nsort()･･･元の変数番号のｿｰﾄ順番号
        'work()･･･ｿｰﾄ変数値
        'xbuf(),ybuf()･･･節点の座標
        Dim i, j, jb, jx, nb, n1, n2, nx As Integer
        Dim b1 As Single
        Dim nsort2(10) As Integer
        Dim work2(10) As Single

        i = 1
        Do
            If work(i) = work(i + 1) Then
                b1 = work(i)
                n1 = i
                For j = i + 1 To Npoint
                    If b1 = work(j) Then
                        n2 = j
                    Else
                        Exit For
                    End If
                Next j
                nx = n2 - n1 + 1
                ReDim nsort2(nx), work2(nx)
                jb = 0
                For j = n1 To n2
                    jb = jb + 1
                    nsort2(jb) = jb
                    nb = nsort(j)
                    If kind = 1 Then
                        work2(jb) = ybuf(nb)
                    Else
                        work2(jb) = xbuf(nb)
                    End If
                Next j
                Call ShellSortStd(nx, nsort2, work2)    'ｼｪｯﾙｿｰﾄ基本
                For j = 1 To nx
                    jb = n1 + nsort2(j) - 1
                    nsort2(j) = nsort(jb)
                Next j
                For j = 1 To nx
                    jx = n1 + j - 1
                    nsort(jx) = nsort2(j)
                Next j
                i = n2 + 1
            Else
                i = i + 1
            End If
        Loop While i < Npoint

    End Sub

    Sub MouseCalc(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            If Kaxis = 0 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                yb = (xa - Xlower) / XYscale + YaxisMin
            End If
        Else
            If Kaxis = 0 Then
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale + XaxisMin
                yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
            End If
        End If

    End Sub

    Sub MouseCalc2()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                If Kaxis = 0 Then
                    xb = (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                    yb = (xa - Xlower) / XYscale + YaxisMin
                End If
            Else
                If Kaxis = 0 Then
                    xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale + XaxisMin
                    yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
                End If
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MousePointSearch(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single)
        'ﾏｳｽ点の節点の探索
        'k1,k2･･･要素の開始、終了番号
        'ky･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点のﾓﾃﾞﾙ座標位置
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        b1 = X(ib) - xb
                        b2 = Y(ib) - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub NumCheck(ByRef ic As Integer, ByRef i1 As Integer, _
        ByRef i2 As Integer, ByRef j1 As Integer, _
        ByRef j2 As Integer)
        '要素辺の重なりのﾁｪｯｸ
        'ic･･･要素辺の重なり有無（0:なし、1:有り）
        'i1･･･第1の辺の一端の節点番号
        'i2･･･第1の辺の他端の節点番号
        'j1･･･第2の辺の一端の節点番号
        'j2･･･第2の辺の他端の節点番号
        ic = 0
        If i1 = j1 Then
            If i2 = j2 Then
                ic = 1
            End If
        ElseIf i1 = j2 Then
            If i2 = j1 Then
                ic = 1
            End If
        End If

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Th2DMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            For i = 1 To 3
                Koption(i) = Val(sr.ReadLine)
            Next i
            Kscreen = Val(sr.ReadLine)
            KcomSide = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Th2DMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 3
            sw.WriteLine(Koption(i))
        Next i
        sw.WriteLine(Kscreen)
        sw.WriteLine(KcomSide)
        sw.Close()

    End Sub

    Sub OutlineDraw(ByRef gr As Graphics, _
        ByVal k1 As Integer, ByVal k2 As Integer)
        '指定要素範囲の外形の作画
        'icheck･･･作画時のXY入れ替えの有無（0:なし、1:有り）
        'k1･･･作画初めの要素番号
        'k2･･･作画終りの要素番号
        Dim i, ie, j, j1, j2, k As Integer
        Dim xi, xj, yi, yj As Single

        Call OutSideCheck()  '外周辺のﾁｪｯｸ

        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    If KEInside(ie, j) = 0 Then
                        For k = 1 To 2
                            If k = 1 Then
                                j1 = 2 * j - 1
                                j2 = j1 + 1
                            Else
                                j1 = j2
                                j2 = j1 + 1
                                If j2 > 8 Then
                                    j2 = 1
                                End If
                            End If
                            i = NNelem(ie, j1)
                            xi = X(i)
                            yi = Y(i)
                            Call XYCalc2(Kaxis, xi, yi)  'X,Yの作画座標計算
                            i = NNelem(ie, j2)
                            xj = X(i)
                            yj = Y(i)
                            Call XYCalc2(Kaxis, xj, yj)  'X,Yの作画座標計算
                            gr.DrawLine(Pens.Black, xi, yi, xj, yj)
                        Next k
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub OutSideCheck()
        '外周辺のﾁｪｯｸ
        Dim i2, ie, ii, j, j2, jj As Integer
        ReDim KEInside(Nelem, 4)

        For ie = 1 To Nelem - 1
            If KdrawElem(ie) > 0 Then
                For j = 1 To 4
                    i2 = NNelem(ie, 2 * j)
                    For ii = ie + 1 To Nelem
                        If KdrawElem(ii) > 0 Then
                            For jj = 1 To 4
                                j2 = NNelem(ii, 2 * jj)
                                If i2 = j2 Then
                                    KEInside(ie, j) = 1
                                    KEInside(ii, jj) = 1
                                End If
                            Next jj
                        End If
                    Next ii
                Next j
            End If
        Next ie

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i) + 0.001
            xydraw(i - 1).Y = Ydraw(i) + 0.001
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PnumImproveNew(ByRef mb As Integer, ByVal mbx As Integer, _
    ByVal xbuf() As Single, ByVal ybuf() As Single)
        '節点付番の改良
        'mb･･･改良後のﾊﾞﾝﾄﾞ幅
        'mbx･･･改良前のﾊﾞﾝﾄﾞ幅
        'xbuf(),ybuf()･･･節点の座標
        Dim i, ie, k, kx, mbb As Integer
        Dim pa, qa, ratio As Single
        Dim kcbuf(Npoint) As Integer

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie
        ratio = 1.2
        i = 0
        Call AxisCheck(i, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        For k = 1 To 2
            Call ShellSort2(k, kcbuf, xbuf, ybuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            If k = 1 Then
                kx = 1
                mbb = mb
            ElseIf mb < mbb Then
                kx = k
                mbb = mb
            End If
        Next k

        If mbb < mbx Then
            Call ShellSort2(kx, kcbuf, xbuf, ybuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            For i = 1 To Npoint
                Kcomp(i) = kcbuf(i)
            Next i
        Else
            mb = mbx
        End If

    End Sub

    Sub PointSearch(ByRef nb As Integer)
        '矩形領域の節点の探索
        'nb･･･適合節点の数
        Dim i As Integer
        Dim b1, b2 As Single
        Dim kbuf(3) As Integer

        nb = 0
        For i = 1 To Npoint
            KAppPoint(i) = 0
            b1 = X(i) - Xmouse(1)
            b2 = X(i) - Xmouse(2)
            If b1 >= 0 And b2 <= 0 Then
                If Kaxis = 0 Then
                    b1 = Y(i) - Ymouse(2)
                    b2 = Y(i) - Ymouse(1)
                Else
                    b1 = Y(i) - Ymouse(1)
                    b2 = Y(i) - Ymouse(2)
                End If
                If b1 >= 0 And b2 <= 0 Then
                    KAppPoint(i) = 1
                    nb = nb + 1
                End If
            End If
        Next i

    End Sub

    Sub QintDupCheck(ByRef nd As Integer)
        '内部発熱の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nqint) As Integer

        nd = 0
        For i = 1 To Nqint - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nqint
                    If Kelemq(i) = Kelemq(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Nstep
                            Qint(i, j) = Qint(ii, j)
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nqint) As Integer
            Dim buf2(Nqint, Nstep) As Single
            For i = 1 To Nqint
                nbuf1(i) = Kelemq(i)
                For j = 1 To Nstep
                    buf2(i, j) = Qint(i, j)
                Next j
            Next i
            nb = Nqint - nd
            ReDim Kelemq(nb), Qint(nb, Nstep)
            nb = 0
            For i = 1 To Nqint
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    Kelemq(nb) = nbuf1(i)
                    For j = 1 To Nstep
                        Qint(nb, j) = buf2(i, j)
                    Next j
                End If
            Next i
            Nqintb = nb
            Nqint = nb
        End If

    End Sub

    Sub RadDupCheck(ByRef nd As Integer)
        '熱放射の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nrad) As Integer

        nd = 0
        For i = 1 To Nrad - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nrad
                    If Kprad(i, 1) = Kprad(ii, 1) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Nstep
                            RadRate(i, j) = RadRate(ii, j)
                            TempRad(i, j) = TempRad(ii, j)
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nrad, 3) As Integer
            Dim buf2(Nrad, Nstep), buf3(Nrad, Nstep) As Single
            For i = 1 To Nrad
                For j = 1 To 3
                    nbuf1(i, j) = Kprad(i, j)
                Next j
                For j = 1 To Nstep
                    buf2(i, j) = RadRate(i, j)
                    buf3(i, j) = TempRad(i, j)
                Next j
            Next i
            nb = Nrad - nd
            ReDim Kprad(nb, 3), RadRate(nb, Nstep), TempRad(nb, Nstep)
            nb = 0
            For i = 1 To Nrad
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 3
                        Kprad(nb, j) = nbuf1(i, j)
                    Next j
                    For j = 1 To Nstep
                        RadRate(nb, j) = buf2(i, j)
                        TempRad(nb, j) = buf3(i, j)
                    Next j
                End If
            Next i
            Nradb = nb
            Nrad = nb
        End If

    End Sub

    Sub RadGDupCheck(ByRef nd As Integer)
        '熱放射ｸﾞﾙｰﾌﾟの重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ic, j, k, kk, mb, mx, nb As Integer
        Dim kbuf(NradG, NumRmax) As Integer

        nd = 0
        For i = 1 To NradG
            For k = 1 To NumRad(i) - 1
                If kbuf(i, k) = 0 Then
                    mb = MemRad(i, k, 2)
                    For kk = k + 1 To NumRad(i)
                        If kbuf(i, kk) = 0 Then
                            mx = MemRad(i, kk, 2)
                            If mb = mx Then
                                kbuf(i, kk) = -1
                                nd = nd + 1
                            End If
                        End If
                    Next kk
                End If
            Next k
        Next i

        If nd > 0 Then
            For i = 1 To NradG
                mx = 0
                For k = 1 To NumRad(i)
                    If kbuf(i, k) = 0 Then
                        mx = mx + 1
                        For j = 1 To 3
                            MemRad(i, mx, j) = MemRad(i, k, j)
                        Next j
                        EpsRad(i, mx) = EpsRad(i, k)
                    End If
                Next k
                NumRad(i) = mx
            Next i
            ic = 0
            For i = 1 To NradG
                If NumRad(i) < 1 Then
                    ic = 1
                End If
            Next i
            If ic > 0 Then
                nb = 0
                For i = 1 To NradG
                    If NumRad(i) > 0 Then
                        nb = nb + 1
                        For k = 1 To NumRad(i)
                            For j = 1 To 3
                                MemRad(nb, k, j) = MemRad(i, k, j)
                            Next j
                            EpsRad(nb, k) = EpsRad(i, k)
                        Next k
                        NumRad(nb) = NumRad(i)
                    End If
                Next i
                NradG = nb
            End If
        End If

        NradGb = NradG

    End Sub

    Sub RGdatAddG(ByVal ngb As Integer)
        '熱放射ｸﾞﾙｰﾌﾟのｸﾞﾙｰﾌﾟ追加
        'ngb･･･元のｸﾞﾙｰﾌﾟ数
        Dim i, j, k, num0, numb As Integer
        Dim nbuf1(ngb), nbuf2(ngb, NumRmax, 3) As Integer
        Dim buf1(ngb), buf3(ngb, NumRmax) As Single

        If ngb > 0 Then
            num0 = NumRmax
            For i = 1 To ngb
                nbuf1(i) = NumRad(i)
                For k = 1 To NumRad(i)
                    For j = 1 To 3
                        nbuf2(i, k, j) = MemRad(i, k, j)
                    Next j
                    buf3(i, k) = EpsRad(i, k)
                Next k
            Next i
            numb = NumRad(1)
            For i = 2 To NradG
                If NumRad(i) > numb Then
                    numb = NumRad(i)
                End If
            Next i
            NumRmax = numb
            If numb < num0 Then
                num0 = numb
            End If
            ReDim NumRad(NradG), MemRad(NradG, NumRmax, 3), EpsRad(NradG, NumRmax)
            For i = 1 To ngb
                NumRad(i) = nbuf1(i)
                For k = 1 To NumRad(i)
                    For j = 1 To 3
                        MemRad(i, k, j) = nbuf2(i, k, j)
                    Next j
                    EpsRad(i, k) = buf3(i, k)
                Next k
            Next i
        Else
            NumRmax = 2
            ReDim NumRad(NradG), MemRad(NradG, NumRmax, 3), EpsRad(NradG, NumRmax)
            For i = 1 To NradG
                NumRad(i) = 2
            Next i
        End If

    End Sub

    Sub RGdatAddG2(ByVal k0 As Integer, ByRef mbuf() As Integer, ByVal epsb As Single)
        '熱放射ｸﾞﾙｰﾌﾟの辺の追加
        'k0･･･ｸﾞﾙｰﾌﾟNo.
        'mbuf()･･･辺の3節点
        'epsb･･･放射率
        Dim i, j, k, num0, numb As Integer
        Dim nbuf1(NradG), nbuf2(NradG, NumRmax, 3) As Integer
        Dim buf1(NradG), buf3(NradG, NumRmax) As Single

        num0 = NumRmax
        For i = 1 To NradG
            nbuf1(i) = NumRad(i)
            For k = 1 To NumRad(i)
                For j = 1 To 3
                    nbuf2(i, k, j) = MemRad(i, k, j)
                Next j
                buf3(i, k) = EpsRad(i, k)
            Next k
        Next i
        numb = NumRad(k0) + 1
        For i = 1 To NradG
            If NumRad(i) > numb Then
                numb = NumRad(i)
            End If
        Next i
        NumRmax = numb
        If numb < num0 Then
            num0 = numb
        End If
        ReDim NumRad(NradG), MemRad(NradG, NumRmax, 3), EpsRad(NradG, NumRmax)
        For i = 1 To NradG
            NumRad(i) = nbuf1(i)
            For k = 1 To NumRad(i)
                For j = 1 To 3
                    MemRad(i, k, j) = nbuf2(i, k, j)
                Next j
                EpsRad(i, k) = buf3(i, k)
            Next k
            If i = k0 Then
                k = NumRad(i) + 1
                NumRad(i) = k
                For j = 1 To 3
                    MemRad(i, k, j) = mbuf(j)
                Next j
                EpsRad(i, k) = epsb
            End If
        Next i

    End Sub

    Sub Rounding(ByVal kind As Integer, ByVal kfnum As Integer, _
        ByRef temp As Single)
        '丸め処理
        'kind･･･丸め種類（0:下丸め、1:上丸め）
        'kfnum･･･小数点以下の桁数
        'temp･･･温度
        Dim b1, b2, b3, b4 As Single
        b1 = 10 ^ kfnum
        b3 = temp * b1
        b4 = Int(b3 / b1)
        b2 = b3 - b1 * b4
        If b2 = 0 Then
            temp = CInt(b3) / b1
        ElseIf kind > 0 Then
            temp = Int(b3 + 1) / b1
        Else
            temp = Int(b3) / b1
        End If

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub ShellSort2(ByVal kind As Integer, ByRef kcbuf() As Integer, _
    ByVal xbuf() As Single, ByVal ybuf() As Single)
        'ｼｪﾙｿｰﾄ
        'kind･･･主ｿｰﾄの座標軸番号（1:X軸、2:Y軸）
        'kcbuf()･･･元の節点の変数順序
        'xbuf(),ybuf()･･･節点の座標
        Dim i As Integer
        Dim nsort(Npoint) As Integer
        Dim work(Npoint) As Single

        Select Case kind
            Case 1
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = xbuf(i)
                Next i
            Case 2
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = ybuf(i)
                Next i
        End Select

        Call ShellSortStd(Npoint, nsort, work)    'ｼｪｯﾙｿｰﾄ基本

        Call MiniSort2(kind, nsort, work, xbuf, ybuf)    'ﾐﾆｿｰﾄ

        For i = 1 To Npoint
            kcbuf(nsort(i)) = i
        Next i

    End Sub

    Sub ShellSortStd(ByVal nx As Integer, ByRef nsort() As Integer, ByRef work() As Single)
        'ｼｪﾙｿｰﾄ基本
        'nx･･･変数の数
        'nsort()･･ｿｰﾄ順の元変数の番号
        'work()･･･ｿｰﾄ順の変数値
        Dim i, ib, j, m As Integer
        Dim varb As Single

        m = nx
        Do While m > 1
            m = m \ 2
            For j = m + 1 To nx
                varb = work(j)
                ib = nsort(j)
                i = j - m
                Do While work(i) >= varb
                    work(i + m) = work(i)
                    nsort(i + m) = nsort(i)
                    i = i - m
                    If i <= 0 Then
                        Exit Do
                    End If
                Loop
                work(i + m) = varb
                nsort(i + m) = ib
            Next j
        Loop

    End Sub

    Private Sub SideSearch(ByRef kh As Integer, ByRef js As Integer,
        ByVal n1 As Integer, ByVal n2 As Integer, ByVal n3 As Integer)
        '辺の探索
        'kh･･･要素番号（0はなし、1:有り）
        'js･･･一致辺の中間節点順序番号
        'n1,n2,n3･･･3節点の番号
        Dim ie, j, jx, k1, k2, k3 As Integer
        kh = 0
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                k1 = NNelem(ie, j)
                If k1 = n1 Then
                    k2 = NNelem(ie, j + 1)
                    If j < 7 Then
                        jx = j + 2
                    Else
                        jx = 1
                    End If
                    k3 = NNelem(ie, jx)
                    If k2 = n2 And k3 = n3 Then
                        kh = ie
                        js = j + 1
                        Exit For
                    End If
                End If
            Next j
            If kh > 0 Then
                Exit For
            End If
        Next ie

    End Sub

    Sub TempComp(ByRef ic As Integer, ByVal b1 As Single, ByRef tempcmax As Single, _
    ByRef tempcmin As Single)
        '温度比較
        'ic･･･初回区分（0:初回、1:次回以降）
        'tempcmax･･･最高温度
        'tempcmin･･･最低温度
        If ic = 0 Then
            tempcmax = b1
            tempcmin = b1
            ic = 1
        ElseIf b1 < tempcmin Then
            tempcmin = b1
        ElseIf b1 > tempcmax Then
            tempcmax = b1
        End If

    End Sub

    Sub TimCal(ByVal ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Therm2D.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Therm2D.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub UFileAddB()
        'UFileBの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileB(i) = UFileB(0) Then
                For j = i - 1 To 0 Step -1
                    UFileB(j + 1) = UFileB(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileB(i + 1) = UFileB(i)
            Next i
        End If
        UseFileBOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileBIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Th2DUFileB.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileB(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileBOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Th2DUFileB.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileB(i) <> "" Then
                sw.WriteLine(UFileB(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub XYCalc2(ByVal icheck As Integer, ByRef xb As Single, _
        ByRef yb As Single)
        'X,Yの作画座標計算
        'icheck･･･作画時のXY入れ替えの有無（0:なし、1:有り）
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            If icheck = 0 Then
                xb = b1 * XYscale + Xlower
                yb = Yupper - b2 * XYscale * Cfix
            Else
                xb = b2 * XYscale + Xlower
                yb = b1 * XYscale * Cfix + Ylower
            End If
        Else
            If icheck = 0 Then
                xb = b1 * XYscale / Cfix + Xlower
                yb = Yupper - b2 * XYscale
            Else
                xb = b2 * XYscale / Cfix + Xlower
                yb = b1 * XYscale + Ylower
            End If
        End If
    End Sub

End Module
